## This program is free software; you can redistribute it and/or modify it under
## the terms of the GNU General Public License as published by the Free Software
## Foundation; either version 3 of the License, or (at your option) any later
## version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
## FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, see <http://www.gnu.org/licenses/>.

.onAttach <- function(libname, pkgname)
{
    packageStartupMessage(
        "NOTE: Packages 'fBasics', 'timeDate', and 'timeSeries' are no longer\n",
        "attached to the search() path when 'fGarch' is attached.\n\n",
        "If needed attach them yourself in your R script by e.g.,\n",
        "        require(\"timeSeries\")")
}
